@@ -9,7 +9,12 @@
 ((window) => {
   const webidl = window.__bootstrap.webidl;
   const { CryptoKey } = window.__bootstrap.crypto;
-  const { ArrayBufferIsView, ArrayBuffer } = window.__bootstrap.primordials;
+  const {
+    ArrayBufferIsView,
+    ArrayBufferPrototype,
+    ObjectPrototypeIsPrototypeOf,
+    SafeArrayIterator,
+  } = window.__bootstrap.primordials;
 
   webidl.converters.AlgorithmIdentifier = (V, opts) => {
     // Union for (object or DOMString)
@@ -21,7 +26,10 @@
 
   webidl.converters["BufferSource or JsonWebKey"] = (V, opts) => {
     // Union for (BufferSource or JsonWebKey)
-    if (ArrayBufferIsView(V) || V instanceof ArrayBuffer) {
+    if (
+      ArrayBufferIsView(V) ||
+      ObjectPrototypeIsPrototypeOf(ArrayBufferPrototype, V)
+    ) {
       return webidl.converters.BufferSource(V, opts);
     }
     return webidl.converters.JsonWebKey(V, opts);
@@ -72,7 +80,7 @@
 
   /** @type {__bootstrap.webidl.Dictionary} */
   const dictRsaKeyGenParams = [
-    ...dictAlgorithm,
+    ...new SafeArrayIterator(dictAlgorithm),
     {
       key: "modulusLength",
       converter: (V, opts) =>
@@ -90,7 +98,7 @@
     .createDictionaryConverter("RsaKeyGenParams", dictRsaKeyGenParams);
 
   const dictRsaHashedKeyGenParams = [
-    ...dictRsaKeyGenParams,
+    ...new SafeArrayIterator(dictRsaKeyGenParams),
     {
       key: "hash",
       converter: webidl.converters.HashAlgorithmIdentifier,
@@ -104,7 +112,7 @@
   );
 
   const dictRsaHashedImportParams = [
-    ...dictAlgorithm,
+    ...new SafeArrayIterator(dictAlgorithm),
     {
       key: "hash",
       converter: webidl.converters.HashAlgorithmIdentifier,
@@ -120,7 +128,7 @@
   webidl.converters.NamedCurve = webidl.converters.DOMString;
 
   const dictEcKeyImportParams = [
-    ...dictAlgorithm,
+    ...new SafeArrayIterator(dictAlgorithm),
     {
       key: "namedCurve",
       converter: webidl.converters.NamedCurve,
@@ -134,7 +142,7 @@
   );
 
   const dictEcKeyGenParams = [
-    ...dictAlgorithm,
+    ...new SafeArrayIterator(dictAlgorithm),
     {
       key: "namedCurve",
       converter: webidl.converters.NamedCurve,
@@ -146,7 +154,7 @@
     .createDictionaryConverter("EcKeyGenParams", dictEcKeyGenParams);
 
   const dictEcImportParams = [
-    ...dictAlgorithm,
+    ...new SafeArrayIterator(dictAlgorithm),
     {
       key: "namedCurve",
       converter: webidl.converters.NamedCurve,
@@ -158,7 +166,7 @@
     .createDictionaryConverter("EcImportParams", dictEcImportParams);
 
   const dictAesKeyGenParams = [
-    ...dictAlgorithm,
+    ...new SafeArrayIterator(dictAlgorithm),
     {
       key: "length",
       converter: (V, opts) =>
@@ -171,7 +179,7 @@
     .createDictionaryConverter("AesKeyGenParams", dictAesKeyGenParams);
 
   const dictHmacKeyGenParams = [
-    ...dictAlgorithm,
+    ...new SafeArrayIterator(dictAlgorithm),
     {
       key: "hash",
       converter: webidl.converters.HashAlgorithmIdentifier,
@@ -188,7 +196,7 @@
     .createDictionaryConverter("HmacKeyGenParams", dictHmacKeyGenParams);
 
   const dictRsaPssParams = [
-    ...dictAlgorithm,
+    ...new SafeArrayIterator(dictAlgorithm),
     {
       key: "saltLength",
       converter: (V, opts) =>
@@ -201,7 +209,7 @@
     .createDictionaryConverter("RsaPssParams", dictRsaPssParams);
 
   const dictRsaOaepParams = [
-    ...dictAlgorithm,
+    ...new SafeArrayIterator(dictAlgorithm),
     {
       key: "label",
       converter: webidl.converters["BufferSource"],
@@ -212,7 +220,7 @@
     .createDictionaryConverter("RsaOaepParams", dictRsaOaepParams);
 
   const dictEcdsaParams = [
-    ...dictAlgorithm,
+    ...new SafeArrayIterator(dictAlgorithm),
     {
       key: "hash",
       converter: webidl.converters.HashAlgorithmIdentifier,
@@ -224,7 +232,7 @@
     .createDictionaryConverter("EcdsaParams", dictEcdsaParams);
 
   const dictHmacImportParams = [
-    ...dictAlgorithm,
+    ...new SafeArrayIterator(dictAlgorithm),
     {
       key: "hash",
       converter: webidl.converters.HashAlgorithmIdentifier,
@@ -350,7 +358,7 @@
   );
 
   const dictHkdfParams = [
-    ...dictAlgorithm,
+    ...new SafeArrayIterator(dictAlgorithm),
     {
       key: "hash",
       converter: webidl.converters.HashAlgorithmIdentifier,
@@ -372,7 +380,7 @@
     .createDictionaryConverter("HkdfParams", dictHkdfParams);
 
   const dictPbkdf2Params = [
-    ...dictAlgorithm,
+    ...new SafeArrayIterator(dictAlgorithm),
     {
       key: "hash",
       converter: webidl.converters.HashAlgorithmIdentifier,
@@ -395,7 +403,7 @@
     .createDictionaryConverter("Pbkdf2Params", dictPbkdf2Params);
 
   const dictAesDerivedKeyParams = [
-    ...dictAlgorithm,
+    ...new SafeArrayIterator(dictAlgorithm),
     {
       key: "length",
       converter: (V, opts) =>
@@ -405,7 +413,7 @@
   ];
 
   const dictAesCbcParams = [
-    ...dictAlgorithm,
+    ...new SafeArrayIterator(dictAlgorithm),
     {
       key: "iv",
       converter: webidl.converters["BufferSource"],
@@ -414,7 +422,7 @@
   ];
 
   const dictAesGcmParams = [
-    ...dictAlgorithm,
+    ...new SafeArrayIterator(dictAlgorithm),
     {
       key: "iv",
       converter: webidl.converters["BufferSource"],
@@ -432,7 +440,7 @@
   ];
 
   const dictAesCtrParams = [
-    ...dictAlgorithm,
+    ...new SafeArrayIterator(dictAlgorithm),
     {
       key: "counter",
       converter: webidl.converters["BufferSource"],
@@ -460,7 +468,7 @@
 
   webidl.converters.CryptoKey = webidl.createInterfaceConverter(
     "CryptoKey",
-    CryptoKey,
+    CryptoKey.prototype,
   );
 
   const dictCryptoKeyPair = [
@@ -478,7 +486,7 @@
     .createDictionaryConverter("CryptoKeyPair", dictCryptoKeyPair);
 
   const dictEcdhKeyDeriveParams = [
-    ...dictAlgorithm,
+    ...new SafeArrayIterator(dictAlgorithm),
     {
       key: "public",
       converter: webidl.converters.CryptoKey,
